Another composable, multi-platform HTTP client

[![pub package](https://img.shields.io/pub/v/roundtripper.svg)](https://pub.dev/packages/roundtripper)
[![ci](https://github.com/morlay/roundtripper/actions/workflows/ci.yaml/badge.svg)](https://github.com/morlay/roundtripper/actions/workflows/ci.yaml)

Totally inspected by [http.RoundTripper](https://pkg.go.dev/net/http#RoundTripper) from golang, and dart style fit.
Based on [dio](https://github.com/flutterchina/dio) for crossing-platforms support.

You could do any processing for each request in `RoundTrip`.

* Logging.
* Throws non-2xx Response as Error.
* Request/Response Body Encoding.

# Using

```dart
import 'package:roundtripper/roundtripper.dart';
import 'package:roundtripper/roundtripbuilders/request_body_convert.dart';
import 'package:roundtripper/roundtripbuilders/request_log.dart';
import 'package:roundtripper/roundtripbuilders/throws_not_2xx_error.dart';

var c = Client(roundTripBuilders: [
  ThrowsNot2xxError(),
  RequestBodyConvert(),
  RequestLog(logger),
]);

void main() async {
  var resp = await c.fetch(Request.uri(
    "$httpBinEndpoint/anything",
    queryParameters: {
      "int": 1,
      "slice": [1, 2],
    },
    headers: {
      "x-int": 1,
      "x-slice": [1, 2],
    },
  ));

  try {
    var json = await resp.json();
  } on ErrorResponse catch (e) {
    // e.response
  }
}
```
