import 'dart:async';
import 'dart:io' show HttpStatus;

import 'response.dart';

/// You can cancel a request by using a cancel token.
/// One token can be shared with different requests.
/// when a token's [cancel] method invoked, all requests
/// with this token will be cancelled.
class CancelToken {
  CancelToken() {
    _completer = Completer<ErrorResponse>();
  }

  /// Whether is throw by [cancel]
  static bool isCancel(ErrorResponse e) {
    return e.statusCode == HttpStatus.clientClosedRequest;
  }

  /// If request have been canceled, save the cancel Error.
  ErrorResponse? _cancelError;

  /// If request have been canceled, save the cancel Error.
  ErrorResponse? get cancelError => _cancelError;

  late Completer<ErrorResponse> _completer;

  /// whether cancelled
  bool get isCancelled => _cancelError != null;

  /// When cancelled, this future will be resolved.
  Future<ErrorResponse> get whenCancel => _completer.future;

  /// Cancel the request
  void cancel([dynamic reason]) {
    _cancelError = ErrorResponse(
      HttpStatus.clientClosedRequest,
      reason: reason,
    );
    if (!_completer.isCompleted) {
      _completer.complete(_cancelError);
    }
  }
}
